﻿using System;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Web.DynamicData;
using System.Web.Routing;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.Expressions;
using Telerik.Web.UI;

public partial class ListDetails : System.Web.UI.Page
{
    protected MetaTable table;
    bool isExport = false;

    protected void RadGrid1_ItemCommand(object sender, GridCommandEventArgs e)
    {
        if (e.CommandName.Contains("Export"))
        {
            if (e.CommandName == "ExportToCsv")
                RadGrid1.ExportSettings.ExportOnlyData = false;

            RadGrid1.MasterTableView.GetColumn("EditColumn").Visible = false;
            RadGrid1.MasterTableView.GetColumn("DeleteColumn").Visible = false;

            isExport = true;
        }
    }

    protected void RadGrid1_ItemDataBound(object sender, GridItemEventArgs e)
    {
        if (e.Item is GridDataItem && isExport)
        {
            GridDataItem item = e.Item as GridDataItem;
            foreach (GridColumn column in RadGrid1.MasterTableView.RenderColumns)
            {
                if (!column.Visible ||
                    column is GridExpandColumn ||
                    column is GridRowIndicatorColumn ||
                    column is GridGroupSplitterColumn)
                    continue;

                foreach (Control ctrl in ((GridDataItem)item)[column.UniqueName].Controls)
                {
                    if (ctrl is DynamicControl)
                    {
                        Control fieldTemplate = (ctrl as DynamicControl).FieldTemplate;
                        Control control = (fieldTemplate as FieldTemplateUserControl).DataControl;
                        string data = string.Empty;
                        if (control is ITextControl)
                            data = (control as ITextControl).Text.Trim();
                        else if (control is HyperLink)
                            data = (control as HyperLink).Text.Trim();
                        else if (control is ICheckBoxControl)
                            data = (control as ICheckBoxControl).Checked.ToString();
                        else if (control is IButtonControl)
                            data = (control as IButtonControl).Text.Trim();
                        if (data != string.Empty)
                        {
                            Control parent = ctrl.Parent;
                            parent.Controls.Remove(ctrl);
                            parent.Controls.Add(new Label() { Text = data });
                        }
                    }
                }
            }
        }
    }

    protected void Page_Init(object sender, EventArgs e)
    {
        table = DynamicDataRouteHandler.GetRequestMetaTable(Context);
        RadGrid1.SetMetaTable(table, table.GetColumnValuesFromRoute(Context));
        FormView1.SetMetaTable(table);
        GridDataSource.EntityTypeName = table.EntityType.AssemblyQualifiedName;
        DetailsDataSource.EntityTypeName = table.EntityType.AssemblyQualifiedName;
        if (table.EntityType != table.RootEntityType)
        {
            GridQueryExtender.Expressions.Add(new OfTypeExpression(table.EntityType));
        }
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        Title = table.DisplayName;

        // Selection from url
        if (!Page.IsPostBack && table.HasPrimaryKey)
        {
            //GridView1.SelectedPersistedDataKey = table.GetDataKeyFromRoute();
            //if(GridView1.SelectedPersistedDataKey == null) {
            //    GridView1.SelectedIndex = 0;
            //}
        }

        // Disable various options if the table is readonly
        if (table.IsReadOnly)
        {
            DetailsPanel.Visible = false;
            //GridView1.AutoGenerateSelectButton = false;
            //GridView1.AutoGenerateEditButton = false;
            //GridView1.AutoGenerateDeleteButton = false;
            //GridView1.EnablePersistedSelection = false;
        }
    }

    protected void GridView1_DataBound(object sender, EventArgs e)
    {
        //if (GridView1.Rows.Count == 0) {
        //    FormView1.ChangeMode(FormViewMode.Insert);
        //}
    }

    protected void Label_PreRender(object sender, EventArgs e)
    {
        Label label = (Label)sender;
        DynamicFilter dynamicFilter = (DynamicFilter)label.FindControl("DynamicFilter");
        QueryableFilterUserControl fuc = dynamicFilter.FilterTemplate as QueryableFilterUserControl;
        if (fuc != null && fuc.FilterControl != null)
        {
            label.AssociatedControlID = fuc.FilterControl.GetUniqueIDRelativeTo(label);
        }
    }

    protected void DynamicFilter_FilterChanged(object sender, EventArgs e)
    {
        RadGrid1.EditIndexes.Clear();
        RadGrid1.CurrentPageIndex = 0;
        FormView1.ChangeMode(FormViewMode.ReadOnly);
    }

    protected void GridView1_RowEditing(object sender, EventArgs e)
    {
        FormView1.ChangeMode(FormViewMode.ReadOnly);
    }

    protected void GridView1_SelectedIndexChanging(object sender, EventArgs e)
    {
        RadGrid1.EditIndexes.Clear();
        FormView1.ChangeMode(FormViewMode.ReadOnly);
    }

    protected void GridView1_RowCreated(object sender, GridViewRowEventArgs e)
    {
        SetDeleteConfirmation(e.Row);
    }

    protected void GridView1_RowDeleted(object sender, GridViewDeletedEventArgs e)
    {
        FormView1.DataBind();
    }

    protected void GridView1_RowUpdated(object sender, GridViewUpdatedEventArgs e)
    {
        FormView1.DataBind();
    }

    protected void FormView1_ItemDeleted(object sender, FormViewDeletedEventArgs e)
    {
        RadGrid1.DataBind();
    }

    protected void FormView1_ItemUpdated(object sender, FormViewUpdatedEventArgs e)
    {
        RadGrid1.DataBind();
    }

    protected void FormView1_ItemInserted(object sender, FormViewInsertedEventArgs e)
    {
        RadGrid1.DataBind();
    }

    protected void FormView1_ModeChanging(object sender, FormViewModeEventArgs e)
    {
        if (e.NewMode != FormViewMode.ReadOnly)
        {
            RadGrid1.EditIndexes.Clear();
        }
    }

    protected void FormView1_PreRender(object sender, EventArgs e)
    {
        if (FormView1.Row != null)
        {
            SetDeleteConfirmation(FormView1.Row);
        }
    }

    private void SetDeleteConfirmation(TableRow row)
    {
        foreach (Control c in row.Cells[0].Controls)
        {
            LinkButton button = c as LinkButton;
            if (button != null && button.CommandName == DataControlCommands.DeleteCommandName)
            {
                button.OnClientClick = "return confirm('Are you sure you want to delete this item?');";
            }
        }
    }

}
